unit Animation_3D;

interface

uses
	{ Borland }
	Windows, SysUtils, Classes, Controls, Graphics, Dialogs,
	{ Downloaded }
	HiResTim,
	{ Mine }
	DIB_surface, Math_Utils, PalUnit, D_Timer, ThinkBox;

type
	pPointStruc = ^PointStruc;
	PointStruc	= record
		x,y,z   	: double;
		c       	: double;
		dx,dy,dz	: double;
		dc      	: double;
	end;
	pPointArray = ^PointArray;
	PointArray  = Array[0..0] of PointStruc;
	pByteArray  = ^ByteArray;
	ByteArray   = Array[0..0] of Byte;

type
	animation__3D = class
		{ data }
		num_frames	: integer;
		pts_frame		: integer;
		data				: pPointArray;
		{ screen variables }
		DIB_surface	: DIBSurfaceObject;
		log_palette : TMyLogPalette;
		xmid,ymid		: integer;
		destination : TWinControl;
		{ palette variables }
		colours			: integer;
		coloursm1		: integer;
		{ 3D variables }
		xrspeed,yrspeed,zrspeed : integer; // ints for lookup tables indices
		xoff,yoff,zoff					: double;
		xscale,yscale						: double;
		{ Matrix values... }
		rotyxz0_0,	rotyxz0_1,  rotyxz0_2,
		rotyxz1_0,  rotyxz1_1,  rotyxz1_2,
		rotyxz2_1,  rotyxz2_2,  rotyxz2_3  : double;
		{ Timing }
		Timer							: THiResTimer;
		callback					: TNotifyEvent;
		frame_sp					: frame_speed_obj;
		active            : boolean;
		{ animation control }
		main_counter			: integer;
		frame_number			:	integer;

		palette_cycle     : integer;
		morph_steps				: integer;
		morph_count				: integer;
		interframe_pause	: integer;
		interframe_count	: integer;
		//
		constructor create(nf,ppf:integer; dest:TWinControl);
		destructor  Destroy; override;
		// screen stuff
		procedure		create_drawing_surface(w,h:integer); virtual;
		procedure		fill_palette_from_screen_colours(use_cols:integer); virtual;
		function		GetPalette : HPalette;
		procedure		animate_palette(num:integer); virtual;
		procedure		Copy_to_screen;
		// enable/disable and timer stuff
		procedure   activate(onoff:boolean);
		procedure   set_frame_speed(fps_:double);
		procedure		Timer_event;
		// initializtion and computation
		procedure		setup_initial_state(i1,i2:integer); 			virtual;
		procedure		update_frame_counters;		virtual;
		procedure 	setup_morphing_constants;
		procedure 	calc_rotation(x,y,z:integer); { unrolled matrix multiply }
		function 		TransformPoint(x,y,z:double) : TPoint;
		procedure		compute_frame;       			virtual; abstract;
		procedure		draw_frame(dest_DC:HDC);	virtual;
		// Getters and Setters
		procedure   set_rot_speed(x,y,z:integer); // ints for lookup tables
		procedure   set_offset(x,y,z:double);
		procedure   set_scale(x,y:double);
		procedure		set_interframepause(d:integer);
		procedure		set_morphsteps(d:integer);
		procedure   set_palette_cycle(pc:integer);
	end;

type
	animated_credits = class(animation__3D)
		a,b,c,d,e,f,g,h,i,j,k,l : integer;
		text_data	: pByteArray;
		pw,ph,ff  : integer;
		constructor load_credits(fname:string; ppf,w,h,f,ff_:integer; dest:TWinControl);
		destructor 	Destroy;															override;
		procedure		fill_palette_from_screen_colours(use_cols:integer); virtual;
		procedure 	create_credit_data;
		procedure		setup_initial_state(i1,i2:integer);		override;
		procedure		compute_frame;       			            override;
		procedure 	draw_frame(dest_DC:HDC);              override;
	end;

type
	animated_colour_cube = class(animation__3D)
		RGB_walker				: pal_animator;
		newcube						: array[0..7] of coord;
		ri,gi,bi					: integer;
		colour_bar_offset	: integer;
		constructor create_colour_cube(dest:TWinControl);
		destructor 	Destroy;											override;
		procedure		animate_palette(num:integer);	override;
		procedure		setup_initial_state(i1,i2:integer);	override;
		procedure		compute_frame;       					override;
		procedure		draw_frame(dest_DC:HDC);			override;
		procedure   Draw_cube;
		procedure		set_RGB_parameters(cp,ecc:integer; gamma:double);
		procedure 	Save_colours_to_File(name:string);
	end;

implementation

const
	root3 = 1.732050807569;

///////////////////////////////////////////////////////////////////////////////
// Animation base class (Abstract object)                                    //
///////////////////////////////////////////////////////////////////////////////
constructor animation__3D.create(nf,ppf:integer; dest:TWinControl);
begin
	num_frames						:= nf;
	pts_frame							:= ppf;
	GetMem(data, num_frames*pts_frame*sizeof(PointStruc));
	Timer									:= THiResTimer.Create(nil);
//	Timer.ThreadPriority	:= tpIdle;
	Timer.Interval				:= 100;
	Timer.OnTimer					:= Timer_event;
	destination						:= dest;
	frame_sp							:= frame_speed_obj.Create;
	setup_tables;
end;

destructor animation__3D.Destroy;
begin
	FreeMem(data, num_frames*pts_frame*sizeof(PointStruc));
	frame_sp.Free;
	Timer.Free;
	inherited destroy;
end;
///////////////////////////
// Screen stuff
///////////////////////////
procedure animation__3D.create_drawing_surface(w,h:integer);
begin
	default_log_palette(log_palette);
	DIB_surface	:= DIBSurfaceobject.Create(log_palette,Point(w,h));
	colours		:=	236;
	coloursm1	:= colours-1;
	xmid			:= w div 2;
	ymid			:= h div 2;
end;

procedure animation__3D.fill_palette_from_screen_colours(use_cols:integer);
var screen	: HDC;
		lp1			: integer;
begin
	Screen := GetDC(0);
	GetSystemPaletteEntries(Screen,10 ,use_cols,log_palette.palEntry[10]);
	if ReleaseDC(0,Screen)=0 then Thinking_error('Screen Device context not released #1');
	for lp1:=10 to 9+use_cols do log_palette.palEntry[lp1].peFlags := PC_RESERVED;
	animatepalette(DIB_surface.DIBhpalette,10,use_cols,@log_palette.palEntry[10]);
	colours		:= use_cols;
	coloursm1	:= colours-1;
end;

function animation__3D.GetPalette : HPalette;
begin
	if DIB_surface<>nil then result := DIB_surface.DIBHPalette
	else result := 0;
end;

procedure animation__3D.animate_palette(num:integer);
var anim_pal : palentries;
begin
	GetPaletteEntries(DIB_surface.DIBhpalette,10,colours,anim_pal[10]);
	if num>0 then begin
		animatepalette(DIB_surface.DIBhpalette,10,colours-num,@anim_pal[10+num]);
		animatepalette(DIB_surface.DIBhpalette,10+colours-num,num,@anim_pal[10]);
	end
	else if num<0 then begin num:=-num;
		animatepalette(DIB_surface.DIBhpalette,10+num,colours-num,@anim_pal[10]);
		animatepalette(DIB_surface.DIBhpalette,10,num,@anim_pal[10+colours-num]);
	end;
end;

procedure animation__3D.Copy_to_screen;
var dest_DC : HDC;
begin
	dest_DC := GetDC(destination.Handle);
	DIB_surface.copy_surface_to_screen(dest_DC);
	if ReleaseDC(destination.Handle,dest_DC)=0 then Thinking_error('Animation Device context not released #1');
end;
///////////////////////////
// Timer stuff
///////////////////////////
procedure animation__3D.activate(onoff:boolean);
begin
  if onoff=active then exit;
	Timer.Enabled := true;
	if onoff then Timer.Resume
	else Timer.Pause;
	active := onoff;
end;

procedure animation__3D.set_frame_speed(fps_:double);
var temp : boolean;
begin
	if not active then begin
		Timer.Pause;
		Timer.Interval	:= round(1000/fps_);
		Timer.Resume;
	end else Timer.Interval	:= round(1000/fps_);
	frame_sp.reset(25,fps_);
end;

procedure animation__3D.Timer_event;
var dest_DC : HDC;
begin
	frame_sp.update(1/tick_seconds_so_far);
	start_ticker;

	animate_palette(palette_cycle);
	update_frame_counters;
	compute_frame;
	dest_DC := GetDC(destination.Handle);
	draw_frame(dest_DC);
	if ReleaseDC(destination.Handle,dest_DC)=0 then Thinking_error('Animation Device context not released #2');
	if assigned(callback) then callback(self);
end;

///////////////////////////
// Init & computation
///////////////////////////
procedure animation__3D.setup_initial_state(i1,i2:integer);
begin
	main_counter	:= 0;
	xrspeed				:= 0;
	yrspeed				:= 0;
	zrspeed				:= 0;
	xoff					:= 0;
	yoff					:= 0;
	zoff					:= 2.9;
	xscale				:= 200;
	yscale				:= 200;

	frame_number			:= 0;
	morph_steps				:= 30;
	morph_count				:= 0;
	interframe_pause	:= 30;
	interframe_count	:= 0;
end;

procedure animation__3D.update_frame_counters;
begin
	inc(main_counter);
	// are we currently pausing between frames
	if (interframe_count>=0) and (interframe_count<interframe_pause) then inc(interframe_count)
	else if (morph_count>=0) and (morph_count<(morph_steps-1)) then inc(morph_count)
	else begin
		interframe_count	:= 0;
		morph_count				:= 0;
		inc(frame_number);
		if frame_number>(num_frames-1) then frame_number := 0;
	end;
	calc_rotation(	(512+xrspeed*main_counter) and tablesize,
									(000+yrspeed*main_counter) and tablesize,
									(000+zrspeed*main_counter) and tablesize );
end;

procedure animation__3D.setup_morphing_constants;
var lp1,lp2,off1,off2	: integer;
		f1,f2  	: integer;
begin
	for lp1:= 0 to num_frames-1 do begin
		f1	 := lp1;
		f2	 := (lp1+1) mod num_frames;
		off1 := f1*pts_frame;
		off2 := f2*pts_frame;
		for lp2:=0 to pts_frame-1 do begin
			with data^[off1 + lp2] do begin
				dx	:= (data^[off2 + lp2].x - x)/ morph_steps;
				dy	:= (data^[off2 + lp2].y - y)/ morph_steps;
				dz	:= (data^[off2 + lp2].z - z)/ morph_steps;
				dc	:= (data^[off2 + lp2].c - c)/ morph_steps;
			end;
		end;
	end;
end;

procedure animation__3D.calc_rotation(x,y,z:integer); { unrolled matrix multiply }
var t1,t2,t3,t4,t5,t6,m1,m2,m3 : double;
begin
	t1:=cosine[x];
	t2:=cosine[y];
	t3:=cosine[z];
	t4:=  sine[x];
	t5:=  sine[y];
	t6:=  sine[z];
	m1 := (t2*t3);
	m2 := (t4*t6);
	m3 := (t3*t5);
	rotyxz0_0 := m1         - ((m2*t5));
	rotyxz0_1 :=-((t1*t5));
	rotyxz0_2 := ((t2*t6))  + ((m3*t4));
	rotyxz1_0 := m3         + ((t2*m2));
	rotyxz1_1 := ((t1*t2));
	rotyxz1_2 := ((t5*t6))  - ((m1*t4));
	rotyxz2_1 :=-((t1*t6));
	rotyxz2_2 := t4;
	rotyxz2_3 := ((t1*t3));
end;

function animation__3D.TransformPoint(x,y,z:double) : TPoint;
var xt,yt,zt : double;
begin
	xt	:=	((rotyxz0_0*x)) +
					((rotyxz0_1*y)) +
					((rotyxz0_2*z)) + xoff;
	yt	:= 	((rotyxz1_0*x)) +
					((rotyxz1_1*y)) +
					((rotyxz1_2*z)) + yoff;
	zt	:= 	((rotyxz2_1*x)) +
					((rotyxz2_2*y)) +
					((rotyxz2_3*z)) + zoff;
	result.x := Round(xscale*(xt/zt) + xmid);
	result.y := Round(ymid - yscale*(yt/zt));
end;

procedure animation__3D.draw_frame(dest_DC:HDC);
var lp1	: integer;
		p1	:	TPoint;
		col	:	integer;
begin
	for lp1:=0 to pts_frame-1 do begin
		with data^[frame_number*pts_frame + lp1] do begin
			p1	:= TransformPoint(x+(dx*morph_count), y+(dy*morph_count), z+(dz*morph_count));
			col := trunc(c+(dc*morph_count));
			if col>9 then DIB_surface.set_pixel( p1.x, p1.y, col );
		end;
	end;
end;
///////////////////////////
// getters and setters
///////////////////////////
procedure animation__3D.set_rot_speed(x,y,z:integer); // ints for lookup tables
begin
	xrspeed := x;
	yrspeed := y;
	zrspeed := z;
end;

procedure animation__3D.set_offset(x,y,z:double);
begin
	xoff := x;
	yoff := y;
	zoff := z;
end;

procedure animation__3D.set_scale(x,y:double);
begin
	xscale := x;
	yscale := y;
end;

procedure animation__3D.set_interframepause(d:integer);
begin
	if interframe_count=interframe_pause then interframe_count:=d;
	interframe_pause := d;
end;

procedure animation__3D.set_morphsteps(d:integer);
begin
	morph_steps := d;
	morph_count := 0;
	setup_morphing_constants;
end;

procedure animation__3D.set_palette_cycle(pc:integer);
begin
	palette_cycle := pc;
end;
///////////////////////////////////////////////////////////////////////////////
// Animated Credits                                                          //
///////////////////////////////////////////////////////////////////////////////
constructor animated_credits.load_credits(fname:string; ppf,w,h,f,ff_:integer; dest:TWinControl);
var charfile	: file;
		lp1,lp2		: integer;
begin
	inherited create(f,ppf,dest);
	pw	:= w;
	ph	:= h;
	ff  := ff_;
	GetMem(text_data, pw*ph*ff);
	if fileexists(fname) then begin
		assignfile(charfile,fname);
		reset(charfile,1);
		blockread(charfile, text_data^, pw*ph*ff);
		closefile(charfile);
	end
	else begin
		ShowMessage('Can''t find '+fname+' data file');
		for lp1:=0 to (pw*ph*ff)-1 do text_data^[lp1] := random(2);
	end;
end;

destructor animated_credits.Destroy;
begin
	FreeMem(text_data, pw*ph*ff);
	inherited destroy;
end;

procedure animated_credits.fill_palette_from_screen_colours(use_cols:integer);
begin
	inherited;
	// to make sure colours wrap properly if number of colours used changes
	create_credit_data;
	setup_morphing_constants;
end;


procedure animated_credits.create_credit_data;
var lp0,lp1,lp2,inx	: integer;
		a,r,cc      		: double;
		r1,r2,cp,sp 		: double;
		i,j,n,m     		: integer;
		points_here 		: integer;
		pp_seg,seg  		: integer;
		phi,theta   		: double;
begin
	points_here	:= 0;
	a	:= 200;
	r	:= pw+pw/12.5;	// frames 0 2 4 on disk, 1 3 5 generated in a minute
	for lp0:=0 to 2 do begin
		points_here	:= 0;
		for lp1:=0 to pw-1 do begin
			for lp2:=0 to ph-1 do begin
				if text_data^[ lp0*pw*ph + ph*(pw-lp1-1) + (ph-lp2-1) ]<>0 then begin
					with data^[lp0*2*pts_frame + points_here] do begin
						x	:= root3*cos(2*pi*(pw-lp1)/r)*cos(pi*((lp2-ph/2)/a));
						z	:= root3*sin(2*pi*(pw-lp1)/r)*cos(pi*((lp2-ph/2)/a));
						y	:= root3*sin(pi*((lp2-ph/2)/a));
						c	:= 10+(lp1 mod colours);
					end;
					inc(points_here);
					if points_here>pts_frame then break;
				end;
			end;
		end; // put the spare points on a tiny sphere
		if points_here<(pts_frame-1) then for lp1:=points_here to (pts_frame-1) do begin
			with data[lp0*2*pts_frame + lp1] do begin
				theta	:= 2*pi*random(256)/255; phi	:= 2*pi*random(256)/255;
				x	:= 0.02*Cos(theta)*Cos(phi);
				z	:= 0.02*Sin(theta)*Cos(phi);
				y	:= 0.02*sin(phi);
				c	:= 9;
			end;
		end;
	end;
	// frame 1 - random cube
	for lp1:=0 to (pts_frame-1) do begin
		with data^[1*pts_frame + lp1] do begin
			x	:= 2*random(256)/255 - 1;
			z	:= 2*random(256)/255 - 1;
			y	:= 2*random(256)/255 - 1;
			c	:= 10+coloursm1*(sqrt(x*x + y*y + z*z)/root3);
		end;
	end;
	// frame 3 - two spheres
	points_here	:= 0;
	seg        	:= 32;
	pp_seg     	:= (pts_frame) div (seg*2);
	m          	:= 0;
	for lp1:=0 to seg-1 do begin
		phi	:= pi*(m+0.5)/seg;
		n  	:= 0;
		for lp2:=0 to pp_seg-1 do begin
			theta	:= 2*pi*(n+0.5)/pp_seg;
			with data^[3*pts_frame + points_here] do begin
				x			:= root3*cos(theta)*sin(phi);
				y			:= root3*sin(theta)*sin(phi);
				z			:= root3*cos(phi);
				c			:= 10+coloursm1*(theta/(2*pi));
				inc(points_here);
			end;
			with data^[3*pts_frame + points_here] do begin
				x			:= root3*0.5*cos(theta)*sin(phi);
				z			:= root3*0.5*sin(theta)*sin(phi);
				y			:= root3*0.5*cos(phi);
				c			:= 10+coloursm1*(theta/(2*pi));
				inc(points_here);
			end;
			n	:= n + 1;
		end;
		m	:= m + 1;
	end;
	// frame 5 - torus * 2
	r1	:= 0.75;
	r2	:= 0.25;
	n 	:= 64;
	m 	:= (pts_frame) div (n*2);
	points_here	:= 0;
	for i:=0 to n-1 do begin
		cp	:= cos(2*pi*i/n);
		sp	:= sin(2*pi*i/n);
//		r2	:= 0.2 + 0.05*cos(8*pi*i/n);	// warp
		for j:=0 to m-1 do begin
			with data^[5*pts_frame + points_here] do begin
				x	:= root3*cp*(r1+r2*cos(2*pi*j/m));
				y	:= root3*sp*(r1+r2*cos(2*pi*j/m));
				z	:= root3*r2*Sin(2*Pi*j/m);
				c	:= 10+coloursm1*((1+cos(2*pi*i/n))/2);
				inc(points_here);
			end;
			with data^[5*pts_frame + points_here] do begin
				x	:= root3*cp*(r1+r2*cos(2*pi*j/m));
				z	:= root3*sp*(r1+r2*cos(2*pi*j/m));
				y	:= root3*r2*Sin(2*Pi*j/m);
				c	:= 10+coloursm1*((1+cos(2*pi*i/n))/2);
				inc(points_here);
			end;
		end;
	end;
end;

procedure animated_credits.setup_initial_state(i1,i2:integer);
begin
	inherited;
	if i1>0 then fill_palette_from_screen_colours(i1)
	else begin
		create_credit_data;
		setup_morphing_constants;
	end;
end;

procedure animated_credits.compute_frame;
begin
end;

procedure animated_credits.draw_frame(dest_DC:HDC);
var lp1,lp2 : integer;
begin
	DIB_surface.clear_surface;
	inherited;
	DIB_surface.copy_surface_to_screen(dest_DC);
end;
///////////////////////////////////////////////////////////////////////////////
// Animated RGB colour cube                                                  //
///////////////////////////////////////////////////////////////////////////////
constructor animated_colour_cube.create_colour_cube(dest:TWinControl);
begin
	inherited create(1,236,dest);
end;

destructor animated_colour_cube.Destroy;
begin
	RGB_Walker.Free;
	inherited;
end;

procedure animated_colour_cube.animate_palette(num:integer);
begin // don't want to animate cube as well !
end;

procedure animated_colour_cube.setup_initial_state;
var lp1 : integer;
begin
	inherited;
	// Palette animator
	RGB_Walker := pal_animator.Create(3,1,1);
	colours    := RGB_Walker.points;
	coloursm1  := colours-1;
	// Cube outline
	for lp1:=0 to 7 do begin
		newcube[lp1,0] := (cube[lp1,0]-0.5)*2;
		newcube[lp1,1] := (cube[lp1,1]-0.5)*2;
		newcube[lp1,2] := (cube[lp1,2]-0.5)*2;
	end;
	ri := GetNearestPaletteIndex(DIB_surface.DIBhpalette,RGB(255,0,0));
	gi := GetNearestPaletteIndex(DIB_surface.DIBhpalette,RGB(0,255,0));
	bi := GetNearestPaletteIndex(DIB_surface.DIBhpalette,RGB(0,0,255));
	// offset for colour scale bar
	colour_bar_offset := (DIB_surface.DIBWidth - colours) div 2;
end;

procedure animated_colour_cube.compute_frame;
var lp1 : integer;
begin
	RGB_walker.update;
	for lp1:=0 to pts_frame-1 do begin
		with data^[lp1] do with RGB_walker.RGB_values[lp1] do begin
			x	:= (2*peRed  /255) - 1;
			z	:= (2*peGreen/255) - 1;
			y	:= (2*peBlue /255) - 1;
			c	:= 10+(lp1 mod colours);
		end;
	end;
	animatepalette(DIB_surface.DIBhpalette,10,colours,@RGB_walker.RGB_values[0]);
end;

procedure animated_colour_cube.draw_frame(dest_DC:HDC);
var lp1,lp2	: integer;
		p1,p2  	: TPoint;
begin
	DIB_surface.clear_surface;
	draw_cube;
	inherited;
	for lp1:= 0 to coloursm1 do for lp2:=0 to 31 do
		DIB_surface.set_pixel(colour_bar_offset+lp1,lp2+5,lp1+10);

	SetBkColor(  DIB_surface.DIBDC, RGB(0,0,0) );
	SetTextColor(DIB_surface.DIBDC, RGB(255,255,255) );
	TextOut(DIB_surface.DIBDC, colour_bar_offset + colours + 10, 14,
		Pchar(IntToStr(colours)),3);
	DIB_surface.copy_surface_to_screen(dest_DC);
end;

procedure animated_colour_cube.Draw_cube;
var lp1		: integer;
		p1,p2 : TPoint;
begin
	for lp1:=0 to 11 do begin
		p1 := TransformPoint(newcube[cube_edges[lp1,0],0], newcube[cube_edges[lp1,0],1], newcube[cube_edges[lp1,0],2]);
		p2 := TransformPoint(newcube[cube_edges[lp1,1],0], newcube[cube_edges[lp1,1],1], newcube[cube_edges[lp1,1],2]);
		if lp1=0 then      DIB_surface.draw_line(p1.x,p1.y,p2.x,p2.y,ri)
		else if lp1=4 then DIB_surface.draw_line(p1.x,p1.y,p2.x,p2.y,gi)
		else if lp1=3 then DIB_surface.draw_line(p1.x,p1.y,p2.x,p2.y,bi)
		else DIB_surface.draw_line(p1.x,p1.y,p2.x,p2.y,255);
	end;
end;

procedure animated_colour_cube.set_RGB_parameters(cp,ecc:integer; gamma:double);
begin
	if RGB_Walker<>nil then RGB_Walker.Free;
	RGB_Walker := pal_animator.Create(cp,ecc,gamma);
	colours    := RGB_Walker.points;
	coloursm1  := colours-1;
	colour_bar_offset := (DIB_surface.DIBWidth - colours) div 2;
end;

procedure animated_colour_cube.Save_colours_to_File(name:string);
var lp1			: integer;
		palfile : Text;
begin
	assignfile(palfile,name);
	rewrite(palfile);
	writeln(palfile,colours);
	for lp1:=0 to coloursm1 do with RGB_Walker.RGB_values[lp1] do begin
		write(palfile, peRed:6, peGreen:6, peBlue:6);
		if lp1=0 then writeln(palfile,
			' Zonerings automatically generated palette : '+DateToStr(Now))
		else writeln(palfile);
	end;
	closefile(palfile);
end;

end.

