unit Form_About;

interface

uses
	{ Borland }
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	Buttons, StdCtrls, ExtCtrls,
	{ Downloaded }
	HiResTim,
	{ Mine }
	Animation_3D, Math_utils, DIB_surface, Palunit, Screen_Utils, D_timer;

type
	TAbout_Form = class(TForm)
		Panel1: TPanel;
		Panel3: TPanel;
		DIBpanel: TPanel;
		Panel2: TPanel;
    RGB_panel: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    eccentricity: TScrollBar;
    Gamma: TScrollBar;
		RGB_params: TRadioGroup;
		credits_panel: TPanel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    z_off: TScrollBar;
    pause_sc: TScrollBar;
    morph_sc: TScrollBar;
    cycle_speed: TScrollBar;
    Panel7: TPanel;
    credits: TPanel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    thank_label: TLabel;
    Panel4: TPanel;
    Panel6: TPanel;
    Panel9: TPanel;
    y_rot: TScrollBar;
    x_rot: TScrollBar;
    z_rot: TScrollBar;
    BntStabilize: TButton;
		about_speed: TPanel;
		OK_btn: TButton;
		Label11: TLabel;
		Label12: TLabel;
		Label13: TLabel;
		swap_btn: TButton;
		palette_info: TPanel;
		Label15: TLabel;
    save_pal_btn: TButton;
    SaveDialog1: TSaveDialog;
    fps: TScrollBar;
    Label10: TLabel;
    Pause_btn: TButton;
		procedure FormShow(Sender: TObject);
		procedure FormHide(Sender: TObject);
		procedure FormDestroy(Sender: TObject);
		// Scrollbar events
		procedure control_change(Sender: TObject);
		procedure RGB_paramsClick(Sender: TObject);
		procedure cred_paramsClick(Sender:TObject);
		// Buttons
		procedure BntStabilizeClick(Sender: TObject);
		procedure swap_btnClick(Sender: TObject);
		procedure OK_btnClick(Sender: TObject);
		procedure save_pal_btnClick(Sender: TObject);
    procedure Pause_btnClick(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
	private
		{ Private declarations }
		function		GetPalette	: hPalette; override;
	public
		{ Public declarations }
		// The display we're looking at
		visible_animation	: animation__3D;
		// the two possible views
		morphing_credits	: animated_credits;
		RGB_Cube					: animated_colour_cube;
		// we need this to load data
		directory	  : string;
		lthanks,thankdelay,thankloop	: integer;
		// and store the idle event handler
		old_idle   	: TIdleEvent;
		procedure		Setup(dir:string; cols:integer);
		procedure		Timer_callback(Sender: TObject);
		procedure 	update_pause;
	end;

var
	About_Form	: TAbout_Form;

implementation

{$R *.DFM}

const
	thanks : string = 'Thanks to Giles, Guy and Tom for suggestions and settings,' +
										' and Lucian for the screensaver help..............';

procedure TAbout_Form.FormCreate(Sender: TObject);
begin
	visible_animation	:= nil;
	RGB_Cube					:= nil;
	morphing_credits	:= nil;
end;

procedure TAbout_Form.Setup(dir:string; cols:integer);
begin
	directory := dir;

	// Create animated credits object and initialize
	if morphing_credits = nil then begin
		morphing_credits	:=	animated_credits.load_credits(
			directory+'Zone_Rings_32_Binary.bin', 2048, 384, 24 , 6, 3, DIBpanel);
		morphing_credits.create_drawing_surface(DIBpanel.Width,DIBpanel.Height);
	end;
	morphing_credits.setup_initial_state(cols,0);
	morphing_credits.callback := Timer_callback;

	// Create an RGB cube object and initialize
	if RGB_Cube = nil then begin
		RGB_Cube := animated_colour_cube.create_colour_cube(DIBpanel);
		RGB_Cube.create_drawing_surface(DIBpanel.Width,DIBpanel.Height);
	end;
	RGB_Cube.setup_initial_state(0,0);
	RGB_Cube.callback := Timer_callback;

	lthanks := length(thanks);
end;

procedure TAbout_Form.FormDestroy(Sender: TObject);
begin
	visible_animation.activate(false);
	RGB_Cube.Free;
	morphing_credits.Free;
end;

procedure TAbout_Form.FormShow(Sender: TObject);
begin
	ScreenCenterWindow(Handle);
	credits_panel.Visible	:= true;
	credits.Visible				:= true;
	RePaint;
	if visible_animation<>nil then visible_animation.activate(false);
	visible_animation			:= morphing_credits;
	control_change(nil);
	cred_paramsClick(nil);
	RGB_paramsClick(nil);
	visible_animation.activate(true);

	thankloop					 	:= 0;
	old_idle						:= Application.OnIdle;
	Application.OnIdle	:= nil;
end;

procedure TAbout_Form.FormHide(Sender: TObject);
begin
  visible_animation.activate(false);
	Application.OnIdle	:= old_idle;
end;

function TAbout_Form.GetPalette	: hPalette;
begin
	if (visible_animation<>nil) then result := visible_animation.GetPalette
	else result	:= inherited GetPalette;
end;

procedure TAbout_Form.Timer_callback(Sender: TObject);
var thanks_str	: string[35];
		lp1					: integer;
begin
	about_speed.Caption	:=
		FloatToStrF(visible_animation.frame_sp.average_frame_speed,ffFixed,5,0);

	inc(thankdelay);
	if (thankdelay>2) then begin
		for lp1:=0 to 34 do thanks_str[lp1+1] := thanks[1+((thankloop+lp1) mod lthanks)];
		thanks_str[0]:=#34;
		thank_label.Caption := thanks_str;
		thankloop := (thankloop+1) mod lthanks;
		thankdelay := 0;
	end;
end;

///////////////////////////////////////////////////////////////////////////////
// Button response routines                                                  //
///////////////////////////////////////////////////////////////////////////////
procedure TAbout_Form.OK_btnClick(Sender: TObject);
begin
	ModalResult	:= mrOK;
	close;
end;

procedure TAbout_Form.RGB_paramsClick(Sender: TObject);
begin
	if (sender=gamma) then begin
		RGB_Cube.RGB_Walker.gamma := Gamma.position;
		RGB_Cube.RGB_Walker.create_pseudo_gamma_lookups(Gamma.position);
	end
	else
		RGB_Cube.set_RGB_parameters(RGB_params.ItemIndex+3,
			eccentricity.position, Gamma.position );
end;

procedure TAbout_Form.cred_paramsClick(Sender:TObject);
begin
	morphing_credits.set_palette_cycle(cycle_speed.position);
	if (sender=pause_sc) or (sender=nil) then morphing_credits.set_interframepause(pause_sc.position);
	if (sender=morph_sc) or (sender=nil) then morphing_credits.set_morphsteps(morph_sc.max+1-morph_sc.position);
end;

procedure TAbout_Form.control_change(Sender: TObject);
begin
	visible_animation.set_frame_speed(fps.position);
	visible_animation.set_rot_speed(	x_rot.position*5,
																		y_rot.position*5,
																		z_rot.position*5);
	visible_animation.set_offset(0,0, 2.9 + z_off.position/50);

end;

procedure TAbout_Form.BntStabilizeClick(Sender: TObject);
begin
	x_rot.position      	:= 0;
	y_rot.position      	:= 0;
	z_rot.position      	:= 10;
	z_off.position      	:= 5;
	cycle_speed.position	:= 5;
	pause_sc.position   	:= 250;
	morph_sc.position   	:= 250;
	fps.position					:= 30;
	control_change(nil);
end;

procedure TAbout_Form.swap_btnClick(Sender: TObject);
begin
	visible_animation.activate(false);
	if visible_animation = RGB_Cube then begin
		credits_panel.Visible	:= true;
		credits.Visible				:= true;
		RePaint;
		visible_animation			:= morphing_credits;
		morphing_credits.fill_palette_from_screen_colours(RGB_Cube.colours);
	end	else begin
		credits_panel.Visible	:= false;
		credits.Visible				:= false;
		RePaint;
		visible_animation			:= RGB_Cube;
		Pause_btn.Caption			:= 'Pause';
	end;
	Control_change(nil);
  PaletteChanged(true);
	visible_animation.activate(true);
	update_pause;
end;

procedure TAbout_Form.save_pal_btnClick(Sender: TObject);
var temp : boolean;
begin
	temp := visible_animation.active;
	visible_animation.activate(false);
	SaveDialog1.InitialDir := Palette_Dir;
	if SaveDialog1.Execute then
		RGB_Cube.Save_colours_to_File(SaveDialog1.Filename);
	visible_animation.activate(temp);
	update_pause;
end;

procedure TAbout_Form.Pause_btnClick(Sender: TObject);
begin
	visible_animation.activate(not visible_animation.active);
	update_pause;
end;

procedure TAbout_Form.update_pause;
begin
	if not visible_animation.active then Pause_btn.Caption := 'Resume'
	else Pause_btn.Caption := 'Pause';
end;

procedure TAbout_Form.FormPaint(Sender: TObject);
begin
	visible_animation.Copy_to_screen;
end;

end.
