///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                          Zonerings Load/Save form                         //
//                             John Biddiscombe                             //
//                           j.biddiscombe@rl.ac.uk                          //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
unit Form_Load_Save;

interface

uses
	{ Borland }
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, Buttons, ComCtrls, Menus, ExtCtrls, Grids,
	{ Mine }
	thinkbox, PalUnit, Screen_Utils, Str_Util, My_LZH, Zone_settings, Zone_Rings,
	Form_Settings, HiResTim, CanvasPanel;

type
	load_save_mode = (LoadMode,ViewMode,SaveMode,MustSaveMode);

type
	TLoadSave_Form = class(TForm)
		Container       : TPanel;
		Info_label      : TLabel;
		ViewButton      : TButton;
		TextView        : TMemo;
    Panel2: TPanel;
    index_panel: TPanel;
    Titles: THeaderControl;
		Can_btn: TButton;
		OK_btn: TButton;
    Screensaver_pic: TImage;
    Name_editor: TEdit;
    settings_list: TListBox;
    Preview_timer: THiResTimer;
    Preview: TPanel;

		procedure FormCreate(Sender: TObject);
		procedure FormDestroy(Sender: TObject);
		procedure FormShow(Sender: TObject);
		procedure FormHide(Sender: TObject);
		// Buttons
		procedure Ok_btnClick(Sender: TObject);
		procedure Can_btnClick(Sender: TObject);
		procedure ViewButtonClick(Sender: TObject);
		// Listbox events
		procedure settings_listDrawItem(Control: TWinControl; Index: Integer;
			Rect: TRect; State: TOwnerDrawState);
		procedure Settings_listMouseDown(Sender: TObject; Button: TMouseButton;
			Shift: TShiftState; X, Y: Integer);
		procedure settings_listMouseUp(Sender: TObject; Button: TMouseButton;
			Shift: TShiftState; X, Y: Integer);
		procedure settings_listKeyUp(Sender: TObject; var Key: Word;
			Shift: TShiftState);
		procedure index_panelMouseDown(Sender: TObject; Button: TMouseButton;
			Shift: TShiftState; X, Y: Integer);
		procedure Name_editorKeyPress(Sender: TObject; var Key: Char);
		procedure Preview_timerTimer;
	private
		{ Private declarations }
	public
		{ Public declarations }

		// main stuff
		Settings_File    : Zone_settings_file;
		Mode             : load_save_mode;
		stored_index     : integer;
		// for drawing items
		rightmargin      : integer;
		// For editing names etc
		oldcaption       : string;
		undo_modified    : boolean;
		editor_active    : boolean;
		// Preview
		Preview_instance   : Zonerings;
		invisible_settings : TSettings_Form;
		preview_index      : integer;
		ready              : boolean;

		// Access to settings stuff
		procedure Open_file(name:string);
		procedure Close_file;
		procedure Save_file;
		function  select_random_setting(just_tagged:boolean) : integer;
		function  isModified : boolean;
		procedure setModified(val:boolean);
		procedure SaveToNewFile(name:string);
		procedure New_file_name(name:string);
		procedure Add_current_setting(name:string);
		procedure Update_list;
		//
		function  Load_setting : integer;
		procedure Save_setting;
		procedure setup_load_mode;
		procedure setup_view_mode;
		procedure setup_save_mode;
		procedure setup_mustsave_mode;
		// Listbox draw
		procedure draw_rect(acanvas:TCanvas; State:TOwnerDrawState; r:TRect);
		// Name editing stuff
		procedure Activate_Editor;
		procedure DeActivate_Editor;
		function 	overwrite_Setting : boolean;
		function  confirm_overwrite : boolean;
		// Preview
		procedure Start_Preview(index:integer);
		procedure Stop_Preview;
		//
		function  GetPalette : hPalette; override;
	end;

var
	LoadSave_Form : TLoadSave_Form;

implementation

{$R *.DFM}

///////////////////////////////////////////////////////////////////////////////
//  Load_Save form file routines
///////////////////////////////////////////////////////////////////////////////
procedure TLoadSave_Form.FormCreate(Sender: TObject);
begin
	ready := false;
	Settings_File      := Zone_settings_file.Create;
	invisible_settings := TSettings_Form.Create(nil);
	with ZOptions do begin
		Open_File(Zone_Directory+Settings_name);
		if (settings_not_found in error_flag) or (settings_corrupt in error_flag) then begin
			settings_list.Items.Clear;
			Add_current_setting('Default unnamed setting');
			Settings_File.Modified := false;
		end;
	end;
	// adjust titles
	rightmargin :=
		index_panel.width - Titles.Sections.Items[1].width - GetSystemMetrics(SM_CXVSCROLL)-2;
	Titles.Sections.Items[0].width := rightmargin;
end;

procedure TLoadSave_form.FormDestroy(Sender: TObject);
begin
	Close_file;
	Settings_File.Free;
	invisible_settings.Free;
	if Preview_instance<>nil then Preview_instance.Free;
end;

procedure TLoadSave_Form.Open_file(name:string);
begin
	Settings_File.Open_file(name,true);
	Update_list;
end;

procedure TLoadSave_Form.Close_file;
begin
	Settings_File.Close_File;
	settings_list.Items.Clear;
end;

procedure TLoadSave_Form.Save_file;
begin
	Settings_File.Save_settings_data;
end;

function TLoadSave_Form.select_random_setting(just_tagged:boolean) : integer;
begin
	result := -1;
	if Settings_File.Entries=0 then exit;
	result := Settings_File.Random_Setting(just_tagged);
	settings_list.ItemIndex := result;
	Settings_Form.ReadText(Settings_File.Entry(result).Control_Strings);
end;

function TLoadSave_Form.isModified : boolean;
begin
	result := Settings_File.Modified;
end;

procedure TLoadSave_Form.setModified(val:boolean);
begin
	Settings_File.Modified := val;
end;

procedure TLoadSave_Form.SaveToNewFile(name:string);
begin
  Settings_File.file_name := name;
  Settings_File.Modified  := true;
	Settings_File.Save_settings_data;
end;

procedure TLoadSave_Form.New_file_name(name:string);
begin
	Settings_File.file_name := name;
end;

procedure TLoadSave_Form.Add_current_setting(name:string);
var z_s : Zone_setting;
begin
	z_s := Zone_setting.Create;
	Settings_Form.WriteText(z_s.Control_strings,name);
	Settings_File.Add(z_s);
	settings_list.Items.Add(name);
end;

procedure TLoadSave_Form.Update_list;
var lp1 : integer;
begin
	settings_list.Items.Clear;
	for lp1:=0 to Settings_File.Entries-1 do
		settings_list.Items.Add(Settings_File.Entry(lp1).name);
end;

///////////////////////////////////////////////////////////////////////////////
//  Load_Save form GUI stuff
///////////////////////////////////////////////////////////////////////////////

function TLoadSave_Form.Load_setting : integer;
begin
	Mode   := LoadMode;
	result := ShowModal;
end;

procedure TLoadSave_Form.Save_setting;
begin
	Mode := SaveMode;
	ShowModal;
end;

procedure TLoadSave_Form.FormShow(Sender: TObject);
var sel,total : integer;
begin
	ScreenCenterWindow(Handle);
	// adjust listbox so that selected setting appears in middle
	with settings_list do begin
		sel := ItemIndex;
		total := Items.Count;
			if (sel-8>=0) and (sel+8<=total) then TopIndex := sel-8
		else if (sel-8<0) then TopIndex := 0
		else if (sel+8>total) then TopIndex := total-15;
	end;
	// make sure title is up to date
	Titles.Sections.Items[0].Text :='Name        ' +
		IntToStr(Settings_File.Entries) + ' Settings in file';
	// store the selected settinf in case user presses cancel later
	stored_index := settings_list.ItemIndex;
	// make sure the preview start off correctly
	preview_index := -1;
	Start_Preview(stored_index);
	Preview.Cursor := 1;
	// conceal the editor box regardless of mode.
	SetWindowPos(name_editor.Handle,HWND_TOP,0,0,0,0,SWP_HIDEWINDOW);
	// setup current mode
	if Mode=LoadMode then setup_load_mode
	else if Mode=SaveMode then setup_save_mode;
end;

procedure TLoadSave_form.FormHide(Sender: TObject);
begin
	Stop_Preview;
end;

procedure TLoadSave_Form.setup_load_mode;
begin
	Mode                := LoadMode;
	TextView.Visible    := false;
	index_panel.Visible := true;
	Info_label.Caption  := 'Click to preview, double click to edit/toggle';
	ViewButton.Caption  := 'View setting as text';
	ViewButton.Enabled  := true;
end;

procedure TLoadSave_Form.setup_view_mode;
const ts : array[0..2] of integer = (64,64+32,64+32+32);
begin
	Mode                  := ViewMode;
	index_panel.Visible   := false;
	TextView.Visible      := true;
	Info_label.Caption    := 'You cannot modify values in this mode';
	ViewButton.Caption    := 'Return to index view';
	SendMessage(TextView.handle,EM_SetTabStops,3,integer(@ts));
end;

procedure TLoadSave_Form.setup_save_mode;
begin
	Mode                := SaveMode;
	Info_label.Caption  := 'You will overwrite selected setting';
	index_panel.Visible := true;
	TextView.Visible    := false;
	ViewButton.Caption  := 'Create new setting';
	ViewButton.Enabled  := true;
	Activate_Editor;
end;

procedure TLoadSave_Form.setup_mustsave_mode;
begin
	Mode                  := MustSaveMode;
	Info_label.Caption    := 'Please enter a suitable (preferably silly) name';
	ViewButton.Enabled    := false;
end;

///////////////////////////////////////////////////////////////////////////////
//  Listbox stuff
///////////////////////////////////////////////////////////////////////////////
procedure TLoadSave_form.draw_rect(acanvas:TCanvas; State:TOwnerDrawState; r:TRect);
begin
	with acanvas do if (odSelected in State) and (not editor_active) then begin
		pen.color   := clHighlight;
		brush.color := clHighlight;
		font.color  := clHighlightText;
		font.style  := [fsBold];
	end else begin
		pen.color   := clBtnFace;
		brush.color := clBtnFace;
		font.color  := clWindowText;
		font.style  := [fsBold];
	end;
	acanvas.FillRect(r);
end;

procedure TLoadSave_form.settings_listDrawItem(Control:TWinControl;
	Index:Integer; Rect:TRect; State:TOwnerDrawState);
var temp,sw : integer;
		tc      : TCanvas;
		box     : TRect;
begin
	tc := (Control as TListBox).Canvas;
	box := Rect;
	// Draw the name in a box
	box.Right := rightmargin-1;
	draw_rect(tc,State,box);
	tc.TextOut(box.Left + 2, box.Top+1, Settings_list.Items[Index]);
	// Draw the bitmap in a box
	box.Right := Rect.Right;
	box.Left  := rightmargin-1;
	draw_rect(tc,State,box);
	if Settings_File.Entry(Index).screensave then begin
		// this stops the image from being colour matched to the cycling colours
		Screensaver_pic.Picture.Bitmap.Palette := GetStockObject(DEFAULT_PALETTE);
		tc.Draw(box.Left+12, Rect.Top+1, Screensaver_pic.picture.bitmap);
	end;
end;

procedure TLoadSave_form.settings_listMouseUp(Sender: TObject;
	Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
	Start_Preview(settings_list.ItemIndex);
	if mode=SaveMode then Activate_Editor;
end;

procedure TLoadSave_form.settings_listKeyUp(Sender: TObject; var Key: Word;
	Shift: TShiftState);
begin
	Start_Preview(settings_list.ItemIndex);
end;

procedure TLoadSave_Form.Settings_listMouseDown(Sender: TObject;
	Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var flag : boolean;
begin
	if (ssDouble in shift) then begin
		if (X>rightmargin) then begin
			flag := Settings_File.Entry(settings_list.ItemIndex).Screensave;
			Settings_File.Set_setting_screensave(settings_list.ItemIndex,not flag);
			settings_listDrawItem(settings_list,settings_list.ItemIndex,
				settings_list.ItemRect(settings_list.ItemIndex),[odSelected]);
		end else Activate_Editor;
	end;
end;

// If the Listbox is disabled, clicks come here....
procedure TLoadSave_form.index_panelMouseDown(Sender: TObject;
	Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var flag  : boolean;
		int	  : integer;
begin
  if not editor_active then exit;
	int := settings_list.ItemIndex;
	flag := PtInRect(settings_list.ItemRect(int), Point(X,Y-Titles.Height));
	if flag then begin
		Settings_File.Set_setting_screensave(int,not Settings_File.Entry(int).Screensave);
		settings_listDrawItem(settings_list,int,settings_list.ItemRect(int),[]);
		exit;
	end;
	if mode=MustSaveMode then exit;
	if PtInRect(settings_list.ClientRect, Point(X,Y-Titles.Height)) then begin
		// MouseClick went into listbox
		DeActivate_Editor;
		if mode=SaveMode then editor_active := true; // avoid repaint highlight
		SendMessage(settings_list.Handle,WM_LBUTTONDOWN,MK_LBUTTON,
			MakeLong(X,Y-Titles.Height));
		SendMessage(settings_list.Handle,WM_LBUTTONUP,MK_LBUTTON,
			MakeLong(X,Y-Titles.Height));
	end else if not PtInRect(Titles.ClientRect,Point(X,Y)) then begin
		// MouseClick went into Scrollbar
		DeActivate_Editor;
	end;
end;

procedure TLoadSave_Form.ViewButtonClick(Sender: TObject);
var astring : string;
		lp1     : integer;
begin
	if Mode=SaveMode then begin
		undo_modified := isModified;
		DeActivate_Editor;
		Add_current_setting('New setting');
		settings_list.ItemIndex := settings_list.Items.Count-1;
		setup_mustsave_mode;
		Activate_Editor;
		Start_Preview(settings_list.ItemIndex); 
	end else if Mode=LoadMode then begin
		TextView.Lines.BeginUpdate;
		TextView.Lines.Clear;
		for lp1:=0 to Settings_File.Entry(settings_list.ItemIndex).control_strings.Count-1 do
			TextView.Lines.Add(Trim(Settings_File.Entry(settings_list.ItemIndex).control_strings[lp1]));
		TextView.Lines.Insert(1,' ');
		TextView.Lines.EndUpdate;
		setup_view_mode;
	end else if Mode=ViewMode then begin
		setup_Load_mode;
	end;
end;

///////////////////////////////////////////////////////////////////////////////
//  Name editing stuff
///////////////////////////////////////////////////////////////////////////////
procedure TLoadSave_form.Activate_Editor;
var Editor_Rect : TRect;
begin
  if settings_list.ItemIndex<0 then exit;
	editor_active := true;
	with settings_list do begin
		Enabled := false;
		name_editor.Text := Items[ItemIndex];
		Editor_Rect := ItemRect(settings_list.ItemIndex);
		with Editor_Rect do
			SetWindowPos(name_editor.Handle,HWND_TOP,
				Left+2,Top+Titles.Height+1,Left+rightmargin-3,Bottom-Top-2,SWP_NOREDRAW);
		ShowWindow(name_editor.Handle,SW_SHOW);
		ActiveControl := name_editor;
		settings_listDrawItem(settings_list,ItemIndex,Editor_Rect,[]);
		if mode=LoadMode then begin
			oldcaption            := Info_label.Caption;
			Info_label.Caption    := 'You must press <Enter> to commit changes';
		end;
	end;
end;

procedure TLoadSave_form.DeActivate_Editor;
begin
	editor_active := false;
	ShowWindow(name_editor.Handle,SW_HIDE);
	settings_list.Enabled := true;
	if mode=LoadMode then Info_label.Caption := oldcaption
end;

procedure TLoadSave_form.Name_editorKeyPress(Sender: TObject; var Key:Char);
var temp : integer;
begin
	if (Key=#13) or (Key=#27) then begin
		if mode=LoadMode then begin
			if (Key=#13) then begin
				Settings_File.Entry(settings_list.ItemIndex).SetName(name_editor.Text);
				temp := settings_list.ItemIndex;
				settings_list.Items[settings_list.ItemIndex] := name_editor.Text;
				settings_list.ItemIndex := temp;
				setModified(true);
			end;
			DeActivate_Editor;
		end
		else if mode=SaveMode then begin
			if (Key=#13) then begin
				if overwrite_setting then ModalResult := mrOk;
			end;
			DeActivate_Editor;
		end;
	end;
end;

function TLoadSave_Form.overwrite_Setting : boolean;
var z_s  : Zone_setting;
		temp : integer;
begin
	if confirm_overwrite then begin
		z_s := Zone_setting.Create;
		Settings_Form.WriteText(z_s.Control_strings,'');
		z_s.SetName(name_editor.Text);
		temp := settings_list.ItemIndex;
		settings_list.Items[temp] := name_editor.Text;
		settings_list.ItemIndex := temp;
		Settings_File.Overwrite(temp,z_s);
		DeActivate_editor;
		result := true;
	end else result := false;
end;

function TLoadSave_Form.confirm_overwrite : boolean;
begin
	case MessageDlg('Overwrite'#13#13+Settings_File.Entry(settings_list.ItemIndex).name,
		mtConfirmation, [mbYes,mbNo], 0) of
		mrYes    : result := true;
		mrNo     : result := false;
	end
end;

procedure TLoadSave_Form.Ok_btnClick(Sender:TObject);
var z_s  : Zone_setting;
begin
	if settings_list.ItemIndex<0 then begin ModalResult := mrCancel; exit; end;
	if (Mode=LoadMode) or (Mode=ViewMode) then begin
		LockWindowUpdate(Settings_Form.handle);
		Settings_Form.ReadText(Settings_File.Entry(settings_list.ItemIndex).control_strings);
		LockWindowUpdate(0);
		ModalResult := mrOK;
	end else if mode=SaveMode then begin
		if overwrite_setting then ModalResult := mrOk;
	end else if mode=MustSaveMode then begin
		z_s := Settings_File.Entry(settings_list.ItemIndex);
		z_s.SetName(name_editor.Text);
		DeActivate_editor;
		settings_list.Items[settings_list.ItemIndex] := name_editor.Text;
		settings_list.ItemIndex := settings_list.Items.Count-1;
		ModalResult := mrOk;
	end;
end;

procedure TLoadSave_Form.Can_btnClick(Sender: TObject);
begin
	if editor_active then DeActivate_editor;
	if mode=MustSaveMode then begin
		Settings_file.Delete(Settings_file.Entries-1);
		Settings_list.Items.Delete(Settings_list.Items.Count-1);
		setModified(undo_modified);
	end;
	settings_list.ItemIndex := stored_index;
	ModalResult := mrCancel;
end;

///////////////////////////////////////////////////////////////////////////////
//  Preview stuff
///////////////////////////////////////////////////////////////////////////////
procedure TLoadSave_form.Start_Preview(index:integer);
begin
	if index=preview_index then exit;
	Stop_Preview;
	if (index<0) or (index>Settings_File.Entries) then exit;
	invisible_settings.ReadText(Settings_File.Entry(index).Control_Strings);
	preview_index					:= index;
	Preview_instance			:= Zonerings.Create(Preview, invisible_settings);
	if Preview_instance.settings.fix_speed.Checked then
		Preview_timer.FPS	 	:= Preview_instance.ideal_speed
	else
		Preview_timer.FPS	 	:= 200;
	ready := true;
	Preview_timer.Enabled	:= true;
end;

procedure TLoadSave_form.Stop_Preview;
begin
	ready := false;
	Preview_timer.Enabled	:= false;
	if Preview_instance<>nil then begin
		Preview_instance.Blank_screen;
		Preview_instance.Free;
		Preview_instance := nil;
	end;
	preview_index := -1;
end;

procedure TLoadSave_form.Preview_timerTimer;
begin
	if not ready then exit;
	Preview_instance.CycleRings(false);
end;

function TLoadSave_Form.GetPalette : hPalette;
begin
	if Preview_instance<>nil then result := Preview_instance.Zone_palette
	else result := inherited GetPalette;
end;

end.
