///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                   Main Zonerings code for visual effects                  //
//                             John Biddiscombe                             //
//                           j.biddiscombe@rl.ac.uk                          //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
//
// have modified pen routines so that they must be created before using this
// code. Pens should be created before, and destroyed after Zonerings objects
// Pens use paletteindex - so colour can be set to anything!
//
// also the default palette must have been setup, just in case the first setting
// used uses palette walking.

unit Zone_Rings;

interface

uses
	{ Borland }
	Windows, Messages, SysUtils, Classes, Controls, Forms, Dialogs, StdCtrls,
	Math,
	{ Mine }
	Math_Utils, PalUnit, Form_Settings, thinkbox, D_timer;

const
	maximum_polys = 500;

type
	polypoints	=	array[0..10] of Tpoint;

const
	buff_size = sizeof(integer)*5 + sizeof(longbool) + sizeof(double)*2;
type
	Pquickcopy = ^quickcopy;
	quickcopy  = array[0..buff_size-1] of byte;

type
	poly	= class(Tobject)
		col				: integer;  // 1 Keep these together for quickcopy
		vel				: integer;	// 2
		xc			 	: integer;	// 3
		yc			 	: integer;  // 4
		sid				: integer;  // 5
		changed		: longbool; // 6
		xasp		 	: double;   // 7
		yasp		 	: double;   // 8
		rad				: double;
		rot				: integer;
		vertices 	: polypoints;
		constructor Create(x,y,s,c,rt,ve:integer; rd,xa,ya:double);
		procedure		erasevert;
		procedure		drawvert;
		procedure		erasepoly;
		procedure		drawpoly;
		procedure		copy_from_below(source:poly);
		procedure		update_vertices;
	end;

	ppolylist = ^polylist;
	polylist	= array[0..maximum_polys] of poly;

	Zonerings	= class(Tobject)
		central_poly	: poly;
		num_polys		 	: integer;

		aspect_flag	 	: boolean;
		aspect_countx : integer;
		aspect_county : integer;
		aspect_incx	 	: integer;
		aspect_incy	 	: integer;

		shape_counter : integer;
		shape_timeout : integer;

		rotat_counter : integer;
		rotat_timeout : integer;
		rot_inc			 	: integer;
		rot_inc2			: integer;
		rot_loop			: integer;
		rot_ampl			: double;
		tot_rot			 	: integer;

		col_inc			 	: integer;
		col_max			 	: integer;

		mot_damping	 	: double;
		movex_timeout : integer;
		movex_counter : integer;
		movey_timeout : integer;
		movey_counter : integer;
		move_tempx		: integer;
		move_tempy		: integer;
		lastx,lasty	 	: integer;
		diffx,diffy	 	: integer;
		x_count			 	: integer;
		x_inc				 	: integer;
		x_count2			: integer;
		x_inc2				: integer;
		x_rad				 	: double;
		x_div				 	: integer;
		y_count			 	: integer;
		y_inc				 	: integer;
		y_count2			: integer;
		y_inc2				: integer;
		y_rad				 	: double;
		y_div				 	: integer;
		listpoly			: polylist;

		xmid,ymid,mmid : integer;
		sigmax,sigmay	: integer;
		x_asp,y_asp		: double;
		total_prob		: integer;					{ all the densities }
		densities		 	: array[0..9] of word;
		mouseX,mouseY : integer;

		Zone_win			: TWinControl;
		old_palette   : HPalette;
		Zone_palette  : integer; // use integer not HPalette for C++Builder compatibility
		log_palette   : TMyLogPalette;
		// colour stuff
		use_palette   : boolean;
		colours       : integer;
		coloursm1     : integer;
		pal_pos			 	: integer;
		pal_inc			 	: integer;
		gamma_correct : double;
		// RGB palette walk & create
		RGB_Walker    : pal_animator;
		ref_pos       : integer;
		// essential !
		Settings			: TSettings_Form;
		// timing
		frame_speed   : frame_speed_obj;
		ideal_speed   : integer;

		constructor Create(Windowed_obj:TWinControl; Settings_:TSettings_Form);
		destructor	destroy;		override;
		procedure		draw_all_rings;
		procedure		CycleRings(paused:boolean);
		function		sides	: word;
		procedure		setup_densities;
		procedure		set_mousepos(x,y:integer);
		procedure		Animate_Palette;
		procedure		Window_Changed(Windowed_obj:TWinControl);
		procedure		Blank_screen;
		procedure		Update_frame_counter;
	end;

var
	// Global pen array - create once only - use paletteindex -
	global_pens         : pen_array;
	// used to make sure palette for about box is OK, even when preview has
	// been activated, with different number of colours to main window.
	Global_colour_value : integer;

implementation

var
	Draw_DC        : hDC;
	Zone_pens	     : pPens;

function log(x:double):double;
begin
	if x>0 then log:=system.ln(x)/system.ln(10)
	else log:=-150;
end;

{ ----------------------------------------------------------------------- }
{                         Polygon object structure                        }
{ ----------------------------------------------------------------------- }
constructor poly.Create(x,y,s,c,rt,ve:integer; rd,xa,ya:double);
begin
	inherited Create;
	xc		 	:= x;
	yc		 	:= y;
	sid			:= s;
	col			:= c;
	rad			:= rd;
	rot			:= rt;
	vel			:= ve;
	xasp	 	:= xa;
	yasp	 	:= ya;
	changed	:= false;
	update_vertices;
end;
{ ----------------------------------------------------------------------- }
procedure poly.update_vertices;
var lp1,theta,thetad	: integer;
begin
	if sid>1 then begin thetad:=theta_d[sid];
		theta:=(rot shr 16) and tablesize;
		for lp1:=0 to sid-1 do begin
			vertices[lp1].x	:= round(xc+(cosine[theta]*xasp*rad));
			vertices[lp1].y	:= round(yc+(sine	[theta]*yasp*rad));
			theta						:= (theta+thetad) and tablesize;
		end;
		vertices[sid].x	:= vertices[0].x;
		vertices[sid].y	:= vertices[0].y;
	end
	else begin
		theta:=(rot shr 16) and tablesize;
		vertices[0].x	:= (xc+round(cosine[theta]*xasp*rad));
		vertices[0].y	:= (yc+round(sine	[theta]*yasp*rad));
	end;
end;

{
	#define PALETTEINDEX(i) /
		((COLORREF) (0x01000000 | (DWORD) (WORD) (i)))
}

procedure poly.erasevert;
var lp1	: integer;
begin
	for lp1:=0 to sid-1 do
		setpixelV(Draw_DC,vertices[lp1].x,vertices[lp1].y, $01000000);
end;

procedure poly.drawvert;
var lp1,colour : smallint;  // must use this for correct bit values
begin
	colour := (col shr 16) + 10;
	for lp1:=0 to sid-1 do
		setpixelV( Draw_DC, vertices[lp1].x, vertices[lp1].y, colour or $01000000);
end;

procedure poly.erasepoly;
begin
	SelectObject(Draw_DC,global_pens[0]);
	if sid>2 then begin
		SelectObject(Draw_DC,global_pens[0]);
		polyline(Draw_DC,vertices,sid+1);
	end else if sid=2 then begin
		SelectObject(Draw_DC,global_pens[0]);
		polyline(Draw_DC,vertices,sid)
	end
	else setpixelV(Draw_DC,vertices[0].x,vertices[0].y, $01000000);
end;

procedure poly.drawpoly;
var colour : smallint;  // must use this for correct bit values
begin
	colour := (col shr 16);
{  // debug
	if (colour<>-10) and ((colour<0) or (colour>235)) then begin
		thinking(nil,Pchar(IntToStr(colour)));
		Sleep(100);
		Stopped_thinking;
	end;
}
	if sid>2 then begin
		SelectObject(Draw_DC,Zone_pens[colour]);
		polyline(Draw_DC,vertices,sid+1);
	end else if sid=2 then begin
		SelectObject(Draw_DC,Zone_pens[colour]);
		polyline(Draw_DC,vertices,sid);
	end
	else setpixelV(Draw_DC,vertices[0].x,vertices[0].y, (colour+10) or $01000000);
end;

procedure poly.copy_from_below(source:poly);
begin  // copy N bytes in one go !
	Pquickcopy(@self.col)^ :=  Pquickcopy(@source.col)^;
	self.rot       := source.rot+source.vel;
	source.changed := false;
	update_vertices;
end;
{ ----------------------------------------------------------------------- }
{          Multiple Polygons (Zonerings) object structure                 }
{ ----------------------------------------------------------------------- }
constructor Zonerings.Create(Windowed_obj:TWinControl; Settings_:TSettings_Form);
var lp1								: integer;
		rad								: double;
		rot,rat						: integer;
		xa,ya							: double;
		scale,dvd,dvd2	 	: double;
		x,y,off						: double;
begin
	Settings	:= Settings_;
	with Settings do begin
		// palette stuff
		use_palette := UsePalette.Checked;
		if use_palette then with palette_list do begin
			if Items.count>0 then begin
				if ItemIndex=-1 then ItemIndex	:= random(Items.count);
				create_256_identity_palette_from_file(log_palette,integer(Zone_palette),Items[ItemIndex]);
				colours   := log_palette.pal_mycolours;
			end;
		end
		else begin
			RGB_Walker := pal_animator.Create(control_points.position, eccentricity.position ,gamma.position);
			colours    := RGB_Walker.points;
			create_default_236_greyscale_palette(log_palette,Zone_palette);
			log_palette.pal_MyColours := colours;
			animatepalette(Zone_palette,10,colours,@RGB_walker.RGB_values[0]);
		end;
		Global_colour_value := colours;
		coloursm1           := colours-1;
		ref_pos             := 0;
		pal_pos		 	:= 0;
		pal_inc		 	:= 5000*longint(Settings.palette_inc.position);
		// Screen (window) setup
		Window_Changed(Windowed_obj);
		// main polygon setup
		setup_densities;
		aspect_countx 	:= 0;
		aspect_county 	:= 0;
		if max_polygons.position>maximum_polys then num_polys:=maximum_polys
		else																				num_polys:=max_polygons.position;
		if num_polys=0 then num_polys	:= 50;
		off		:= (vanishing.position) / (vanishing.max+1);
		dvd		:= 4/magnification.max;
		scale	:= mmid*dvd*magnification.position;
		dvd		:= (1-off)/num_polys;
		dvd2 	:= 5 / perspective.max;
		x_asp	:= ((aspect_a_x.position) / (aspect_a_x.max));
		y_asp	:= ((aspect_a_y.position) / (aspect_a_y.max));
		aspect_incx	 	:= aspect_f_x.position*8;
		aspect_incy	 	:= aspect_f_y.position*8;
		if (aspect_f_x.position<>0) or (aspect_f_y.position<>0) then aspect_flag:=true
		else aspect_flag	:= false;
		xa	:= x_asp;
		ya	:= y_asp;
		for lp1:=0 to num_polys do begin
			rot:=0; rat:=0;
			if in_out.checked then x	:= off + lp1*dvd
			else									 x	:= off + (num_polys-lp1)*dvd;
			if aspect_flag then begin
				xa	:= 1-(x_asp*(0.5+(sine[aspect_countx] / 2)));
				ya	:= 1-(y_asp*(0.5+(sine[aspect_county] / 2)));
			end;
			if perspective.position>=0 then begin
				y		:= 1 + (dvd2*perspective.position);
				rad	:= round(powerxy(x,y)*scale);
			end else begin
				y		:=-1 + dvd2*perspective.position;
				rad	:= round((1-powerxy(1-x,-y))*scale);
			end;
			listpoly[lp1]:=poly.Create(xmid,ymid,sides,0, rot, rat, rad, xa, ya);
		end;

		listpoly[0].col	:= 65536*random(coloursm1);
		central_poly	 	:= listpoly[0];

		if shape_per.checked then shape_timeout	:= shape_freq.max-shape_freq.position
		else											shape_timeout	:= round(50*(2-Log(shape_freq.max-shape_freq.position)));
		shape_counter 	:= 0;

		rotat_counter 	:= 0;
		rot_inc					:= (8*precess.position) shl 16;
		rot_inc2				:= (4*freq.position);
		rot_loop				:= 0;
		rot_ampl				:= (2*ampl.position);
		tot_rot					:= max_rot_speed.position-min_rot_speed.position;
		if rot_ran_per.checked then rotat_timeout	:= rot_ran_freq.max-rot_ran_freq.position
		else												rotat_timeout	:= round(50*(2-Log(rot_ran_freq.max-rot_ran_freq.position)));
		if ran_rotation.checked then with central_poly do begin
			rot	:= random(tablesizep1) shl 16;
			vel	:= (random(tot_rot)+min_rot_speed.position)*(8+random(8)) shl 16;
		end;

		col_inc			 	:= 6000*integer(colour_step.position);
		col_max			 	:= 65536*coloursm1;

		movex_counter 	:= 0;
		movey_counter 	:= 0;
		diffx				 		:= 0;			diffy					:= 0;
		mouseX					:= xmid;	mouseY			 	:= ymid;
		if no_motion.checked then begin end
		else if mouse_motion.checked then begin
			mot_damping		:= (400/xmid)*(1-((mot_b_damping.position) / (mot_b_damping.max+1)));
		end else if ran_fluid_motion.checked then begin
			mot_damping		:= (400/xmid)*(((mot_b_damping.position) / (mot_b_damping.max+1)));
			movex_timeout	:= 0;
			movey_timeout	:= 0;
		end else if ran_jump_motion.checked then begin
			movex_timeout	:= mot_ran_freq.max-mot_ran_freq.position
		end else if par_motion.checked then with central_poly do begin
			x_count 	:= 0;
			x_inc		 	:= 2*mot_par_a.position;
			x_count2	:= 0;
			x_inc2		:= 2*mot_par_b.position;
			x_rad		 	:= mot_par_c.position / mot_par_c.max;
			y_count 	:= 0;
			y_inc		 	:= 2*mot_par_d.position;
			y_count2	:= 0;
			y_inc2		:= 2*mot_par_e.position;
			y_rad			:= mot_par_f.position / mot_par_f.max;
			xc	:= xmid + round((xmid*x_rad)*(cosine[x_count]*cosine[x_count2]));
			yc	:= ymid + round((ymid*y_rad)*(sine	[y_count]*cosine[y_count2]));
		end;

	end;

	// Initially - set colours to -10 (=black)
	for lp1:=1 to num_polys do listpoly[lp1].col := -655360;
	Zone_pens := @global_pens[10]; // just a pointer offset by 10

	// timing
	frame_speed  := frame_speed_obj.Create;
	ideal_speed  := Settings.speed_scroll.position;
	frame_speed.reset(25, ideal_speed);

end;

destructor Zonerings.destroy;
var lp1	: word;
begin
	if RGB_walker<>nil then RGB_walker.destroy;
	if frame_speed<>nil then frame_speed.destroy;
	for lp1:=0 to num_polys do listpoly[lp1].destroy;
	if not DeleteObject(Zone_palette) then Thinking_error('Palette not deleted #-1');
	inherited destroy;
end;

procedure Zonerings.draw_all_rings;
var lp1			: word;
		Draw_DC	: HDC;
begin
	Draw_DC := GetDC(Zone_win.Handle);
	if Draw_DC=0 then Thinking_error('Device context invalid #1');
	if not Settings.draw_vertices.checked then
		for lp1:=num_polys downto 1 do listpoly[lp1].drawpoly
	else
		for lp1:=num_polys downto 1 do listpoly[lp1].drawvert;
	if ReleaseDC(Zone_win.Handle,Draw_DC)=0 then Thinking_error('Device context not released #2');
end;

procedure Zonerings.CycleRings(paused:boolean);
var lp1 	: integer;
		rubbingout,aspectsync	: boolean;
begin
	Draw_DC := GetDC(Zone_win.Handle);
	if Draw_DC=0 then Thinking_error('Device context invalid #3');

	SelectPalette(Draw_DC,Zone_palette,false);
	RealizePalette(Draw_DC);
	animate_palette;

	if not paused then with Settings do with central_poly do begin
		rubbingout	:= rub_out.checked;
		aspectsync	:= aspect_sync.checked;
		if not draw_vertices.checked then begin
			for lp1:=num_polys downto 1 do begin
				if (rubbingout and listpoly[lp1-1].changed) then listpoly[lp1].erasepoly;
				listpoly[lp1].copy_from_below(listpoly[lp1-1]);
				if aspectsync then begin
					listpoly[lp1].xasp 	:= central_poly.xasp;
					listpoly[lp1].yasp 	:= central_poly.yasp;
				end;
				listpoly[lp1].drawpoly;
			end;
		end else begin
			for lp1:=num_polys downto 1 do begin
				if (rubbingout and listpoly[lp1-1].changed) then listpoly[lp1].erasevert;
				listpoly[lp1].copy_from_below(listpoly[lp1-1]);
				if aspectsync then begin
					listpoly[lp1].xasp 	:= central_poly.xasp;
					listpoly[lp1].yasp 	:= central_poly.yasp;
				end;
				listpoly[lp1].drawvert;
			end;
		end;

		{ do we need to update the innermost/outermost zonering }

		if aspect_flag then begin
			aspect_countx 	:= (aspect_countx+aspect_incx) and tablesize;
			aspect_county 	:= (aspect_county+aspect_incy) and tablesize;
			xasp 	:= 1-(x_asp*(0.5+(sine[aspect_countx] / 2)));
			yasp 	:= 1-(y_asp*(0.5+(sine[aspect_county] / 2)));
			changed	:= true;
		end;

		if shape_per.checked then begin
			inc(shape_counter);
			if shape_counter>shape_timeout then begin
				shape_counter	:= 0;
				sid						:= sides;
				changed				:= true;
			end;
		end else if random(100)<shape_timeout then begin
			sid			:= sides;
			changed	:= true;
		end;

		if no_rotation.checked then begin end
		else if sin_rotation.checked then begin
			rot		 	:= rot + rot_inc;
			rot_loop	:=(rot_loop + rot_inc2) and tablesize;
			vel		 	:= round(sine[rot_loop]*rot_ampl) shl 16;
			changed 	:= true;
		end
		else begin
			if rot_ran_per.checked then begin
				inc(rotat_counter);
				if rotat_counter>rotat_timeout then begin
					rotat_counter	:= 0;
					rot	:= random(tablesizep1) shl 16;
					vel	:= (random(tot_rot)+min_rot_speed.position)*(8+random(8)) shl 16;
					changed	:= true;
				end;
			end else if rot_ran_ran.checked then begin
				if random(100)<rotat_timeout then begin
					rot	:= random(tablesizep1) shl 16;
					vel	:= (random(tot_rot)+min_rot_speed.position)*(8+random(8)) shl 16;
					changed	:= true;
				end;
			end;
		end;

		// check for blanked out polygons (on startup)
		if col = -655360 then begin end // -655360 = -10
		else begin
			col := col + col_inc;
			if col>col_max then col := col-col_max
			else if col<0  then col := col+col_max;
		end;

		if no_motion.checked then begin end
		else if mouse_motion.checked then begin
			xc		 	:= round(xc+((mouseX-xc)*mot_damping));
			yc		 	:= round(yc+((mouseY-yc)*mot_damping));
			changed	:= true;
		end else if ran_fluid_motion.checked then begin
			inc(movex_counter);
			if movex_counter>movex_timeout then begin
				lastx					:= xc;
				if ran_uniform.checked then diffx	:=(random(2*xmid))-xc
				else												diffx	:=xmid-xc + (round(RandG(0,sigmax))*ranbit);
				movex_timeout	:= system.Abs(round(diffx*mot_damping));
				if movex_timeout>0 then begin
					movex_counter	:= 0;
					move_tempx	 	:= tablesize div movex_timeout;
				end else begin
					movex_counter	:= 1;
					move_tempx	 	:= tablesize;
				end;
			end;
			inc(movey_counter);
			if movey_counter>movey_timeout then begin
				lasty					:= yc;
				if ran_uniform.checked then diffy	:=(random(2*ymid))-yc
				else												diffy	:=ymid-yc + (round(RandG(0,sigmay))*ranbit);
				movey_timeout	:= system.Abs(round(diffy*mot_damping));
				if movey_timeout>0 then begin
					movey_counter	:= 0;
					move_tempy	 	:= tablesize div movey_timeout;
				end else begin
					movey_counter	:= 1;
					move_tempy	 	:= tablesize;
				end;
			end;
			changed	:= true;
			xc	:= lastx + round(diffx*sigmoids[movex_counter*move_tempx and tablesize]);
			yc	:= lasty + round(diffy*sigmoids[movey_counter*move_tempy and tablesize]);
		end
		else if ran_jump_motion.checked then begin
			inc(movex_counter);
			if movex_counter>movex_timeout then begin
				movex_counter	:= 0;
				if ran_uniform.checked then begin
					xc	:= (random(2*xmid));
					yc	:= (random(2*ymid));
				end else begin
					xc	:= xmid-(round(RandG(0,sigmax))*ranbit);
					yc	:= ymid-(round(RandG(0,sigmay))*ranbit);
				end;
				changed				:= true;
			end;
		end else if par_motion.checked then begin
			x_count 	:= (x_count+x_inc) and tablesize;
			y_count 	:= (y_count+y_inc) and tablesize;
			x_count2	:= (x_count2+x_inc2) and tablesize;
			y_count2	:= (y_count2+y_inc2) and tablesize;
			xc	:= xmid + round((xmid*x_rad)*(cosine[x_count]*cosine[x_count2]));
			yc	:= ymid + round((ymid*y_rad)*(sine	[y_count]*cosine[y_count2]));
			changed 	:= true;
		end;
	end;

	if ReleaseDC(Zone_win.Handle,Draw_DC)=0 then Thinking_error('Device context not released #4');
end;

function Zonerings.sides	: word;
var lp1,prob	: word;
begin
	prob:=random(total_prob)+1;
	for lp1:=0 to 9 do begin
		if prob<=densities[lp1] then begin sides:=lp1+1; exit; end
	end;
	sides:=0;
end;

procedure Zonerings.setup_densities;
var lp1	: Integer;
begin
	total_prob:=0;
	for lp1:=0 to 9 do begin
		total_prob	 	:= total_prob -
			TScrollbar(Settings.FindComponent('density'+IntToStr(lp1+1))).position;
		densities[lp1]:= total_prob;
	end;
end;

procedure Zonerings.set_mousepos(x,y:integer);
begin
	mouseX	:= x;
	mouseY	:= y;
end;

// NB : the palette animation still uses an increment based on the old fixed
// point arithmetic implementation (hence the funny 65536 etc etc)
procedure Zonerings.Animate_Palette;
var anim_pal : palentries;
		temp     : integer;
begin
	pal_pos  := pal_pos + pal_inc;
	temp     := smallint(HiWord(pal_pos));
	pal_pos  := pal_pos - 65536*temp;

	if use_palette then begin
		if temp=0 then exit;
		GetPaletteEntries(Zone_palette,10,colours,anim_pal[10]);
		if temp>0 then begin
			animatepalette(Zone_palette,10,colours-temp,@anim_pal[10+temp]);
			animatepalette(Zone_palette,10+colours-temp,temp,@anim_pal[10])
		end
		else begin temp:=-temp;
			animatepalette(Zone_palette,10+temp,colours-temp,@anim_pal[10]);
			animatepalette(Zone_palette,10,temp,@anim_pal[10+colours-temp])
		end;
	end
	else begin
		RGB_walker.update;
		ref_pos := ref_pos + temp;
		if ref_pos<0 then ref_pos := ref_pos+colours
		else if ref_pos>coloursm1 then ref_pos:=ref_pos-colours;
		animatepalette(Zone_palette,10,colours-ref_pos,@RGB_walker.RGB_values[ref_pos]);
		animatepalette(Zone_palette,10+colours-ref_pos,ref_pos,@RGB_walker.RGB_values[0])
	end;
end;

procedure Zonerings.Window_Changed(Windowed_obj:TWinControl);
var win_rect	: TRect;
		lp1		 	: integer;
begin
	Zone_win  := Windowed_obj;
	win_rect := Zone_win.ClientRect;
	Xmid	:= ((win_rect.BottomRight.X-1) div 2);
	ymid	:= ((win_rect.BottomRight.Y-1) div 2);
	sigmax	:= (xmid) div 4;
	sigmay	:= (ymid) div 4;
	if xmid>ymid then mmid:=ymid
	else							mmid:=xmid;
	Blank_screen;
end;

procedure Zonerings.Blank_screen;
var win_rect	: TRect;
		Draw_DC		: HDC;
begin
	Draw_DC := GetDC(Zone_win.Handle);
	if Draw_DC=0 then Thinking_error('Device context invalid #5');
	win_rect := Zone_win.ClientRect;
	PatBlt(Draw_DC,0,0,win_rect.BottomRight.X,win_rect.BottomRight.Y,BLACKNESS);
	if ReleaseDC(Zone_win.Handle,Draw_DC)=0 then Thinking_error('Device context not released #6');
end;

procedure Zonerings.Update_frame_counter;
begin
	frame_speed.update(1/tick_seconds_so_far);
	start_ticker;
end;

{
// Routines used in (integer) fixed point version - not implemented here.

function Fixed16_16ToFloat(x:integer)	: extended;
begin
	Fixed16_16ToFloat:=x/65536;
end;

function FloatToFixed16_16(x:extended)	: integer;
begin
	FloatToFixed16_16:=round(x*65536);
end;

function Fixed16_16Mul(A,B:integer)	: integer; register; assembler;
asm // A = EAX	: B = EDX	: C = ECX
	IMUL EDX;
	SHRD EAX,EDX,16;
end;
{
function Fixed16_16Mul3(A,B,C:integer)	: integer; register; assembler;
asm // A = EAX	: B = EDX	: C = ECX
	IMUL EDX;
	SHRD EAX,EDX,16;
	IMUL ECX;
	SHRD EAX,EDX,16;
end;
}

initialization
	Draw_DC := 0;
finalization
end.
